"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const worker_threads_1 = require("worker_threads");
const hatchet_client_1 = require("../../hatchet-client");
const config_loader_1 = require("../../../util/config-loader");
const nice_grpc_1 = require("nice-grpc");
const dispatcher_client_1 = require("../dispatcher-client");
const HEARTBEAT_INTERVAL = 4000;
class HeartbeatWorker {
    constructor(config, workerId) {
        this.timeLastHeartbeat = new Date().getTime();
        this.workerId = workerId;
        this.logger = new hatchet_client_1.HatchetLogger(`Heartbeat`, config.log_level);
        const credentials = config_loader_1.ConfigLoader.createCredentials(config.tls_config);
        const clientFactory = (0, nice_grpc_1.createClientFactory)().use((0, hatchet_client_1.addTokenMiddleware)(config.token));
        const dispatcher = new dispatcher_client_1.DispatcherClient(Object.assign(Object.assign({}, config), { logger: (ctx, level) => new hatchet_client_1.HatchetLogger(ctx, level) }), (0, hatchet_client_1.channelFactory)(config, credentials), clientFactory);
        this.client = dispatcher.client;
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.heartbeatInterval) {
                return;
            }
            const beat = () => __awaiter(this, void 0, void 0, function* () {
                try {
                    this.logger.debug('Heartbeat sending...');
                    yield this.client.heartbeat({
                        workerId: this.workerId,
                        heartbeatAt: new Date(),
                    });
                    const now = new Date().getTime();
                    const actualInterval = now - this.timeLastHeartbeat;
                    if (actualInterval > HEARTBEAT_INTERVAL * 1.2) {
                        this.logger.warn(`Heartbeat interval delay (${actualInterval}ms >> ${HEARTBEAT_INTERVAL}ms)`);
                    }
                    this.logger.debug(`Heartbeat sent ${actualInterval}ms ago`);
                    this.timeLastHeartbeat = now;
                }
                catch (e) {
                    if (e.code === nice_grpc_1.Status.UNIMPLEMENTED) {
                        // break out of interval
                        this.logger.error('Heartbeat not implemented, closing heartbeat');
                        this.stop();
                        return;
                    }
                    this.logger.error(`Failed to send heartbeat: ${e.message}`);
                }
            });
            // start with a heartbeat
            yield beat();
            this.heartbeatInterval = setInterval(beat, HEARTBEAT_INTERVAL);
        });
    }
    stop() {
        if (this.heartbeatInterval) {
            clearInterval(this.heartbeatInterval);
            this.heartbeatInterval = null;
        }
    }
}
const heartbeat = new HeartbeatWorker(worker_threads_1.workerData.config, worker_threads_1.workerData.workerId);
heartbeat.start();
worker_threads_1.parentPort === null || worker_threads_1.parentPort === void 0 ? void 0 : worker_threads_1.parentPort.on('stop', () => {
    heartbeat.stop();
});
