import { EventClient } from '../event/event-client';
import { DispatcherClient } from '../dispatcher/dispatcher-client';
import { AdminClient } from '../admin/admin-client';
import { CallOptions, ChannelCredentials, ClientMiddlewareCall } from 'nice-grpc';
import { Workflow as V0Workflow } from '../../workflow';
import { V0Worker, WorkerOpts } from '../worker';
import { AxiosRequestConfig } from 'axios';
import { Logger } from '../../util/logger';
import { WorkflowDeclaration as V1Workflow } from '../../v1/workflow';
import { ClientConfig } from './client-config';
import { ListenerClient } from '../listener/listener-client';
import { Api } from '../rest/generated/Api';
import { CronClient } from './features/cron-client';
import { ScheduleClient } from './features/schedule-client';
export interface HatchetClientOptions {
    config_path?: string;
    credentials?: ChannelCredentials;
}
export declare const channelFactory: (config: ClientConfig, credentials: ChannelCredentials) => import("nice-grpc").Channel;
export declare const addTokenMiddleware: (token: string) => <Request, Response>(call: ClientMiddlewareCall<Request, Response>, options: CallOptions) => AsyncGenerator<Awaited<Response>, Awaited<Response> | undefined, undefined>;
export declare class InternalHatchetClient {
    config: ClientConfig;
    credentials: ChannelCredentials;
    event: EventClient;
    dispatcher: DispatcherClient;
    admin: AdminClient;
    api: Api;
    listener: ListenerClient;
    tenantId: string;
    logger: Logger;
    cron: CronClient;
    schedule: ScheduleClient;
    constructor(config?: Partial<ClientConfig>, options?: HatchetClientOptions, axiosOpts?: AxiosRequestConfig);
    static init(config?: Partial<ClientConfig>, options?: HatchetClientOptions, axiosConfig?: AxiosRequestConfig): InternalHatchetClient;
    run(workflow: string | V0Workflow): Promise<V0Worker>;
    worker(workflow: string | V0Workflow, opts?: Omit<WorkerOpts, 'name'> | number): Promise<V0Worker>;
    webhooks(workflows: Array<V1Workflow<any, any> | V0Workflow>): import("../worker/handler").WebhookHandler;
}
