"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncDelegator = (this && this.__asyncDelegator) || function (o) {
    var i, p;
    return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
    function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: false } : f ? f(v) : v; } : f; }
};
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = Object.create((typeof AsyncIterator === "function" ? AsyncIterator : Object).prototype), verb("next"), verb("throw"), verb("return", awaitReturn), i[Symbol.asyncIterator] = function () { return this; }, i;
    function awaitReturn(f) { return function (v) { return Promise.resolve(v).then(f, reject); }; }
    function verb(n, f) { if (g[n]) { i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; if (f) i[n] = f(i[n]); } }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.InternalHatchetClient = exports.addTokenMiddleware = exports.channelFactory = void 0;
const zod_1 = require("zod");
const config_loader_1 = require("../../util/config-loader");
const event_client_1 = require("../event/event-client");
const dispatcher_client_1 = require("../dispatcher/dispatcher-client");
const admin_client_1 = require("../admin/admin-client");
const nice_grpc_1 = require("nice-grpc");
const worker_1 = require("../worker");
const hatchet_logger_1 = require("./hatchet-logger");
const worker_2 = require("../../v1/client/worker");
const client_config_1 = require("./client-config");
const listener_client_1 = require("../listener/listener-client");
const rest_1 = __importDefault(require("../rest"));
const cron_client_1 = require("./features/cron-client");
const schedule_client_1 = require("./features/schedule-client");
const channelFactory = (config, credentials) => (0, nice_grpc_1.createChannel)(config.host_port, credentials, {
    'grpc.ssl_target_name_override': config.tls_config.server_name,
    'grpc.keepalive_timeout_ms': 60 * 1000,
    'grpc.client_idle_timeout_ms': 60 * 1000,
    // Send keepalive pings every 10 seconds, default is 2 hours.
    'grpc.keepalive_time_ms': 10 * 1000,
    // Allow keepalive pings when there are no gRPC calls.
    'grpc.keepalive_permit_without_calls': 1,
});
exports.channelFactory = channelFactory;
const addTokenMiddleware = (token) => function _(call, options) {
    return __asyncGenerator(this, arguments, function* _1() {
        var _a, e_1, _b, _c;
        const optionsWithAuth = Object.assign(Object.assign({}, options), { metadata: new nice_grpc_1.Metadata({ authorization: `bearer ${token}` }) });
        if (!call.responseStream) {
            const response = yield __await(yield* __asyncDelegator(__asyncValues(call.next(call.request, optionsWithAuth))));
            return yield __await(response);
        }
        try {
            for (var _d = true, _e = __asyncValues(call.next(call.request, optionsWithAuth)), _f; _f = yield __await(_e.next()), _a = _f.done, !_a; _d = true) {
                _c = _f.value;
                _d = false;
                const response = _c;
                yield yield __await(response);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (!_d && !_a && (_b = _e.return)) yield __await(_b.call(_e));
            }
            finally { if (e_1) throw e_1.error; }
        }
        return yield __await(undefined);
    });
};
exports.addTokenMiddleware = addTokenMiddleware;
class InternalHatchetClient {
    constructor(config, options, axiosOpts) {
        // Initializes a new Client instance.
        // Loads config in the following order: config param > yaml file > env vars
        var _a;
        const loaded = config_loader_1.ConfigLoader.loadClientConfig(config, {
            path: options === null || options === void 0 ? void 0 : options.config_path,
        });
        try {
            const valid = client_config_1.ClientConfigSchema.parse(loaded);
            let logConstructor = config === null || config === void 0 ? void 0 : config.logger;
            if (logConstructor == null) {
                logConstructor = hatchet_logger_1.DEFAULT_LOGGER;
            }
            this.config = Object.assign(Object.assign({}, valid), { logger: logConstructor });
        }
        catch (e) {
            if (e instanceof zod_1.z.ZodError) {
                throw new Error(`Invalid client config: ${e.message}`);
            }
            throw e;
        }
        this.credentials =
            (_a = options === null || options === void 0 ? void 0 : options.credentials) !== null && _a !== void 0 ? _a : config_loader_1.ConfigLoader.createCredentials(this.config.tls_config);
        const clientFactory = (0, nice_grpc_1.createClientFactory)().use((0, exports.addTokenMiddleware)(this.config.token));
        this.tenantId = this.config.tenant_id;
        this.api = (0, rest_1.default)(this.config.api_url, this.config.token, axiosOpts);
        this.event = new event_client_1.EventClient(this.config, (0, exports.channelFactory)(this.config, this.credentials), clientFactory);
        this.dispatcher = new dispatcher_client_1.DispatcherClient(this.config, (0, exports.channelFactory)(this.config, this.credentials), clientFactory);
        this.listener = new listener_client_1.ListenerClient(this.config, (0, exports.channelFactory)(this.config, this.credentials), clientFactory, this.api);
        this.admin = new admin_client_1.AdminClient(this.config, (0, exports.channelFactory)(this.config, this.credentials), clientFactory, this.api, this.tenantId, this.listener);
        this.logger = this.config.logger('HatchetClient', this.config.log_level);
        this.logger.debug(`Initialized HatchetClient`);
        // Feature Clients
        this.cron = new cron_client_1.CronClient(this.tenantId, this.config, this.api, this.admin);
        this.schedule = new schedule_client_1.ScheduleClient(this.tenantId, this.config, this.api, this.admin);
    }
    static init(config, options, axiosConfig) {
        return new InternalHatchetClient(config, options, axiosConfig);
    }
    // @deprecated
    run(workflow) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.warn('HatchetClient.run is deprecated and will be removed in a future release. Use HatchetClient.worker and Worker.start instead.');
            const worker = yield this.worker(workflow);
            worker.start();
            return worker;
        });
    }
    worker(workflow, opts) {
        return __awaiter(this, void 0, void 0, function* () {
            const name = typeof workflow === 'string' ? workflow : workflow.id;
            let options = {
                name,
            };
            if (typeof opts === 'number') {
                this.logger.warn('@deprecated maxRuns param is deprecated and will be removed in a future release in favor of WorkerOpts');
                options = Object.assign(Object.assign({}, options), { maxRuns: opts });
            }
            else {
                options = Object.assign(Object.assign({}, options), opts);
            }
            const worker = new worker_1.V0Worker(this, options);
            if (typeof workflow !== 'string') {
                yield worker.registerWorkflow(workflow);
                return worker;
            }
            return worker;
        });
    }
    webhooks(workflows) {
        const worker = new worker_1.V0Worker(this, {
            name: 'webhook-worker',
        });
        return worker.getHandler(workflows.map(worker_2.toV0Workflow));
    }
}
exports.InternalHatchetClient = InternalHatchetClient;
