import { IncomingMessage, ServerResponse } from 'http';
import { Workflow } from '../../workflow';
import { V0Worker } from './worker';
export interface HandlerOpts {
    secret: string;
}
export declare class WebhookHandler {
    private worker;
    private workflows;
    constructor(worker: V0Worker, workflows: Workflow[]);
    /**
     * Handles a request with a provided body, secret, and signature.
     *
     * @param {string | undefined} body - The body of the request.
     * @param {string | undefined} secret - The secret used for signature verification.
     * @param {string | string[] | undefined | null} signature - The signature of the request.
     *
     * @throws {HatchetError} - If no signature is provided or the signature is not a string.
     * @throws {HatchetError} - If no secret is provided.
     * @throws {HatchetError} - If no body is provided.
     */
    handle(body: string | undefined, signature: string | string[] | undefined | null, secret: string | undefined): Promise<void>;
    private checkSignature;
    private getHealthcheckResponse;
    /**
     * Express Handler
     *
     * This method is an asynchronous function that returns an Express middleware handler.
     * The handler function is responsible for handling incoming requests and invoking the
     * corresponding logic based on the provided secret.
     */
    expressHandler({ secret }: HandlerOpts): (req: any, res: any) => void;
    /**
     * A method that returns an HTTP request handler.
     */
    httpHandler({ secret }: HandlerOpts): (req: IncomingMessage, res: ServerResponse) => void;
    /**
     * A method that returns a Next.js pages router request handler.
     */
    nextJSPagesHandler({ secret }: HandlerOpts): (req: any, res: any) => Promise<any>;
    /**
     * A method that returns a Next.js request handler.
     */
    nextJSHandler({ secret }: HandlerOpts): {
        GET: () => Promise<Response>;
        POST: (req: Request) => Promise<Response>;
        PUT: (req: Request) => Promise<Response>;
    };
    private getBody;
}
