"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DedupeViolationErr = void 0;
const nice_grpc_1 = require("nice-grpc");
const dispatcher_1 = require("../protoc/dispatcher");
class DedupeViolationErr extends Error {
    constructor(message) {
        super(message);
        this.name = 'DedupeViolationErr';
    }
}
exports.DedupeViolationErr = DedupeViolationErr;
function getWorkflowRunId(workflowRunId) {
    return __awaiter(this, void 0, void 0, function* () {
        if (typeof workflowRunId === 'string') {
            return workflowRunId;
        }
        if (workflowRunId instanceof Promise) {
            try {
                const resolved = yield workflowRunId;
                if (typeof resolved === 'string') {
                    return resolved;
                }
                return resolved.workflowRunId;
            }
            catch (e) {
                if (e.code && e.code === nice_grpc_1.Status.ALREADY_EXISTS) {
                    throw new DedupeViolationErr(e.details);
                }
                throw e;
            }
        }
        throw new Error('Invalid workflowRunId: must be a string or a promise');
    });
}
class WorkflowRunRef {
    constructor(workflowRunId, client, parentWorkflowRunId) {
        this.workflowRunId = workflowRunId;
        this.parentWorkflowRunId = parentWorkflowRunId;
        this.client = client;
    }
    getWorkflowRunId() {
        return __awaiter(this, void 0, void 0, function* () {
            return getWorkflowRunId(this.workflowRunId);
        });
    }
    stream() {
        return __awaiter(this, void 0, void 0, function* () {
            const workflowRunId = yield getWorkflowRunId(this.workflowRunId);
            return this.client.stream(workflowRunId);
        });
    }
    result() {
        return __awaiter(this, void 0, void 0, function* () {
            const workflowRunId = yield getWorkflowRunId(this.workflowRunId);
            const streamable = yield this.client.get(workflowRunId);
            return new Promise((resolve, reject) => {
                (() => __awaiter(this, void 0, void 0, function* () {
                    var _a, e_1, _b, _c;
                    var _d, _e;
                    try {
                        for (var _f = true, _g = __asyncValues(streamable.stream()), _h; _h = yield _g.next(), _a = _h.done, !_a; _f = true) {
                            _c = _h.value;
                            _f = false;
                            const event = _c;
                            if (event.eventType === dispatcher_1.WorkflowRunEventType.WORKFLOW_RUN_EVENT_TYPE_FINISHED) {
                                if (event.results.some((r) => !!r.error)) {
                                    reject(event.results);
                                    return;
                                }
                                if (event.results.length === 0) {
                                    const data = yield this.client.api.workflowRunGetShape(this.client.config.tenant_id, event.workflowRunId);
                                    const mostRecentJobRun = (_d = data.data.jobRuns) === null || _d === void 0 ? void 0 : _d[0];
                                    if (!mostRecentJobRun) {
                                        reject(new Error('No job runs found'));
                                        return;
                                    }
                                    const outputs = {};
                                    (_e = mostRecentJobRun.stepRuns) === null || _e === void 0 ? void 0 : _e.forEach((stepRun) => {
                                        var _a, _b;
                                        const readable = (_b = (_a = mostRecentJobRun.job) === null || _a === void 0 ? void 0 : _a.steps) === null || _b === void 0 ? void 0 : _b.find((step) => step.metadata.id === stepRun.stepId);
                                        const readableStepName = `${readable === null || readable === void 0 ? void 0 : readable.readableId}`;
                                        try {
                                            outputs[readableStepName] = JSON.parse(stepRun.output || '{}');
                                        }
                                        catch (error) {
                                            outputs[readableStepName] = stepRun.output;
                                        }
                                    });
                                    resolve(outputs);
                                    return;
                                }
                                const result = event.results.reduce((acc, r) => (Object.assign(Object.assign({}, acc), { [r.stepReadableId]: JSON.parse(r.output || '{}') })), {});
                                resolve(result);
                                return;
                            }
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (!_f && !_a && (_b = _g.return)) yield _b.call(_g);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                }))();
            });
        });
    }
    toJSON() {
        return __awaiter(this, void 0, void 0, function* () {
            return JSON.stringify({
                workflowRunId: yield this.workflowRunId,
            });
        });
    }
}
exports.default = WorkflowRunRef;
